@@ -78,7 +78,8 @@ def PrepareTeacherData(data_in: Union[np.ndarray, torch.Tensor],
                        trainingLength: int, 
                        testingLength: int, 
                        esn_start: int, 
-                       esn_end: int) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor]:
+                       esn_end: int,
+                       validationLength: int = None) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor]:
     ''' Prepares the input and output training and testing/validation data set for the teacher forced ESN case, i.e. where the
         reservoir input is a teacher signal. 
 
@@ -89,6 +90,7 @@ def PrepareTeacherData(data_in: Union[np.ndarray, torch.Tensor],
         n_output       - output dimensions/ no. output modes
         trainingLength - no. time steps for the training data set
         testingLength  - no. time steps for the testing/validation data set
+        validationLength -  no. time steps for the validation data set
         esn_start      - Index of the original data, at which the training output y_train will begin. 
         esn_end        - Index of the original data, at which the testing/validation output y_test will end.
 
@@ -97,6 +99,8 @@ def PrepareTeacherData(data_in: Union[np.ndarray, torch.Tensor],
         y_train - training output data set. Shape: (trainingLength, n_output)
         u_test  - testing/validation input data set.  Shape: (testingLength, n_input)
         y_test  - testing/validation output data set. Shape: (testingLength, n_output)
+        u_val   - validation input data set (not used in auotnomous predictor mode). Shape: (validationLength, n_input)
+        y_val   - validation output data set. Shape: (validationLength, n_output)
     '''
     data_in_esn = data_in[esn_start-1:esn_end,:n_input]
     data_out_esn = data_out[esn_start-1:esn_end,:n_output]
@@ -107,7 +111,15 @@ def PrepareTeacherData(data_in: Union[np.ndarray, torch.Tensor],
     u_test = data_in_esn[trainingLength:trainingLength+testingLength]
     y_test = data_out_esn[trainingLength:trainingLength+testingLength]
 
-    return torch.as_tensor(u_train, dtype = _DTYPE), torch.as_tensor(y_train, dtype = _DTYPE),torch.as_tensor(u_test, dtype = _DTYPE),torch.as_tensor(y_test, dtype = _DTYPE)
+    if validationLength is not None:
+        u_val = data_in_esn[trainingLength+testingLength:trainingLength+testingLength+validationLength]
+        y_val = data_out_esn[trainingLength+testingLength:trainingLength+testingLength+validationLength]
+
+    else:
+        u_val = None
+        y_val = None
+
+    return torch.as_tensor(u_train, dtype = _DTYPE), torch.as_tensor(y_train, dtype = _DTYPE),torch.as_tensor(u_test, dtype = _DTYPE),torch.as_tensor(y_test, dtype = _DTYPE), torch.as_tensor(u_val, dtype = _DTYPE),torch.as_tensor(y_val, dtype = _DTYPE)
 
 #--------------------------------------------------------------------------
 def Recursion(iparam: int, iterators: np.ndarray, study_parameters: tuple):
